import {
  APIRequest,
  APIDecoder,
  APIEncoder,
  ParameterUtil
} from '@ucap-webmessenger/api';
import { DeviceType } from '@ucap-webmessenger/core';
import { PromptAPIResponse } from './api-prompt';

export interface ConferenceCreateRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  targetUserSeqs: number[];
}

export interface ConferenceCreateResponse extends PromptAPIResponse {}

const conferenceCreateEncodeMap = {
  userSeq: 'userSeq',
  deviceType: 'deviceType',
  tokenKey: 'tokenKey',
  targetUserSeqs: 'targetUserSeqs'
};

export const encodeConferenceCreate: APIEncoder<ConferenceCreateRequest> = (
  req: ConferenceCreateRequest
) => {
  const extraParams: any = {};
  extraParams.targetUserSeqs = req.targetUserSeqs.join(',');

  return ParameterUtil.encode(conferenceCreateEncodeMap, req, extraParams);
};

export const decodeConferenceCreate: APIDecoder<ConferenceCreateResponse> = (
  res: any
) => {
  return {
    responseCode: res.responseCode,
    responseMsg: res.responseMsg
  } as ConferenceCreateResponse;
};
