import { createReducer, on } from '@ngrx/store';
import { initialState, adapterUserInfo, adapterUserInfoShort } from './state';
import {
  infoSuccess,
  updateSuccess,
  updateRoomUserLastReadSeq,
  updateTimeRoomIntervalSuccess,
  clearRoomUser
} from './actions';

import * as AuthenticationStore from '@app/store/account/authentication';
import * as ChatStore from '@app/store/messenger/chat';
import { UserInfo, UserInfoShort } from '@ucap-webmessenger/protocol-room';

export const reducer = createReducer(
  initialState,
  on(infoSuccess, (state, action) => {
    return {
      ...state,
      roomInfo: action.roomInfo,
      userInfoList: adapterUserInfo.addAll(action.userInfoList, {
        ...state.userInfoList
      }),
      userInfoShortList: adapterUserInfoShort.addAll(action.userInfoShortList, {
        ...state.userInfoShortList
      })
    };
  }),

  on(clearRoomUser, (state, action) => {
    const tmpUserInfoList: UserInfo[] = [];
    const tmpUserInfoShortList: UserInfoShort[] = [];
    action.userSeqs.forEach(userSeq => {
      const userInfo: UserInfo = {
        ...state.userInfoList.entities[userSeq]
      };

      if (!!userInfo && !!userInfo.seq) {
        tmpUserInfoList.push({
          ...userInfo,
          isJoinRoom: false
        });
      }

      const userInfoShort: UserInfoShort = {
        ...state.userInfoShortList.entities[userSeq]
      };
      if (!!userInfoShort && !!userInfoShort.seq) {
        tmpUserInfoShortList.push({
          ...userInfoShort,
          isJoinRoom: false
        });
      }
    });

    return {
      ...state,
      userInfoList:
        tmpUserInfoList.length > 0
          ? adapterUserInfo.upsertMany(tmpUserInfoList, {
              ...state.userInfoList
            })
          : state.userInfoList,
      userInfoShortList:
        tmpUserInfoShortList.length > 0
          ? adapterUserInfoShort.upsertMany(tmpUserInfoShortList, {
              ...state.userInfoShortList
            })
          : state.userInfoShortList
    };
  }),

  on(updateSuccess, (state, action) => {
    const curRoomInfo = state.roomInfo;
    if (!curRoomInfo) {
      return { ...state };
    }

    let roomName = curRoomInfo.roomName;
    let receiveAlarm = curRoomInfo.receiveAlarm;

    if (curRoomInfo.roomSeq === action.res.roomSeq) {
      roomName = action.res.roomName;
      receiveAlarm = action.res.receiveAlarm;
    }

    return {
      ...state,
      roomInfo: {
        ...state.roomInfo,
        roomName,
        receiveAlarm
      }
    };
  }),

  on(updateTimeRoomIntervalSuccess, (state, action) => {
    return {
      ...state,
      roomInfo: {
        ...state.roomInfo,
        timeRoomInterval: action.res.timerInterval
      }
    };
  }),

  on(updateRoomUserLastReadSeq, (state, action) => {
    const userSeq = action.SENDER_SEQ;

    const userInfo: UserInfo = {
      ...state.userInfoList.entities[userSeq],
      lastReadEventSeq: action.lastReadSeq
    };

    return {
      ...state,
      userInfoList: adapterUserInfo.updateOne(
        { id: userSeq, changes: userInfo },
        { ...state.userInfoList }
      )
    };
  }),

  on(ChatStore.clearSelectedRoom, (state, action) => {
    return {
      ...initialState
    };
  }),

  on(AuthenticationStore.logoutInitialize, (state, action) => {
    return {
      ...initialState
    };
  })
);
